"""
Wrapper for git commands
"""
import subprocess


def _get_latest_commit_hash(src_path, package_name):
    """
    Get the latest commit hash where the package_name was last modified
    This assumes a folder structure like:
    src/package_name/__version__.py

    Args:
        src_path: The path to the src folder
        package_name: The name of the package
    Returns:

    """

    try:
        # get latest commit where the module was changed excluding the __version__.py file
        git_command = f'git log -1 --pretty=format:%H -- {src_path}/{package_name} ":^{src_path}/{package_name}/__version__.py"'
        result = subprocess.run(git_command, shell=True, capture_output=True, text=True, check=True)
        result_ = result.stdout.split("\n")[0].strip()
        return result_
    except subprocess.CalledProcessError as e:
        print(f"Error: {e}")
        return None
    except Exception as e:
        print(f"Error: {e}")
        return None


def has_code_changed(source_folder: str, package_name: str, latest_hash: str) -> bool | str:
    """
    Check if the code of the package has been modified since last saved commit.
    Compares the latest git commit hash with the saved commit hash.
    If they don't match then there's been changes in the code.


    Args:
        source_folder: the path to the source folder
        package_name: package name
        latest_hash: the latest commit hash saved


    Returns:
        commit_hash: str - The latest commit hash if the code has changed
        False: bool - If the code has not changed

    """

    latest_commit_hash = _get_latest_commit_hash(source_folder, package_name)
    if latest_hash and latest_hash == latest_commit_hash:
        return False
    else:
        return latest_commit_hash



from typing import Literal

from .utils.constants import SUPPORTED_BUMP_TYPES, VERSION_METADATA_FILE
from .utils.git import has_code_changed as _has_code_changed
from .utils.metadata import (load_version_metadata as _load_version_metadata,
                             save_version_metadata as _save_version_metadata)
from .utils.version import (parse_version as _parse_version, get_version_file_path as _get_version_file_path,
                            bump_version as _bump_version, save_bump as _save_bump)


def bump_packages_if_modified(src_folder, packages: list[str], bump_level: SUPPORTED_BUMP_TYPES = 'patch',
                              versions_json_file=VERSION_METADATA_FILE):
    version_metadata = _load_version_metadata(versions_json_file)

    for package in packages:
        print(f"Checking and bumping {package}")

        # Check if the package has been modified
        if latest_commit_hash := _has_code_changed(src_folder, package, version_metadata):
            print(f"Changes detected in package '{package}'.")

            # Parse current version
            version_file_path = _get_version_file_path(src_folder, package)
            version = _parse_version(version_file_path)
            print(f"Current version: {version}")

            # Bump version
            bumped_version = _bump_version(version, bump_level)
            print("Bumping...")
            print(f"Bumped version: {bumped_version}")

            # Save bumped version metadata
            version_metadata[package] = {
                "version": str(bumped_version),
                "commit_hash": latest_commit_hash
            }

            # Save bumped version into the __version__.py file
            _save_bump(version_file_path, bumped_version)
            print(f"Version successfully bumped to {bumped_version} for package {package}")
        else:
            print(f"No changes detected in package '{package}'.")

    print("Saving versions metadata")
    _save_version_metadata(data=version_metadata, to=versions_json_file)
